/// <reference types="node" />
/// <reference types="node" />
import { Readable } from 'node:stream';
import { IncomingHttpHeaders } from 'node:http';
export type SocketInfo = {
    id: number;
    localAddress: string;
    localPort: number;
    remoteAddress: string;
    remotePort: number;
    remoteFamily: string;
    bytesWritten: number;
    bytesRead: number;
    handledRequests: number;
    handledResponses: number;
};
/**
 * https://eggjs.org/en/core/httpclient.html#timing-boolean
 */
export type Timing = {
    queuing: number;
    connected: number;
    requestHeadersSent: number;
    requestSent: number;
    waiting: number;
    contentDownload: number;
};
export type RawResponseWithMeta = Readable & {
    status: number;
    statusCode: number;
    statusText: string;
    headers: IncomingHttpHeaders;
    timing: Timing;
    socket: SocketInfo;
    size: number;
    aborted: boolean;
    rt: number;
    keepAliveSocket: boolean;
    requestUrls: string[];
};
export type HttpClientResponse<T = any> = {
    opaque: unknown;
    data: T;
    status: number;
    statusCode: number;
    statusText: string;
    headers: IncomingHttpHeaders;
    url: string;
    redirected: boolean;
    requestUrls: string[];
    res: RawResponseWithMeta;
};
